@@ -4,6 +4,7 @@ from __future__ import annotations
 from .adv_parser import SwitchbotSupportedType, parse_advertisement_data
 from .const import SwitchbotModel
 from .devices.bot import Switchbot
+from .devices.bulb import ColorMode, SwitchbotBulb
 from .devices.curtain import SwitchbotCurtain
 from .devices.device import SwitchbotDevice
 from .devices.plug import SwitchbotPlugMini
@@ -14,6 +15,8 @@ __all__ = [
     "parse_advertisement_data",
     "GetSwitchbotDevices",
     "SwitchBotAdvertisement",
+    "ColorMode",
+    "SwitchbotBulb",
     "SwitchbotDevice",
     "SwitchbotCurtain",
     "Switchbot",
